@@ -182,9 +182,10 @@ type Config struct {
 	homeDirAbsPath chezmoi.AbsPath
 	encryption     chezmoi.Encryption
 
-	stdin  io.Reader
-	stdout io.Writer
-	stderr io.Writer
+	stdin       io.Reader
+	stdout      io.Writer
+	stderr      io.Writer
+	bufioReader *bufio.Reader
 
 	tempDirs map[string]chezmoi.AbsPath
 
@@ -434,6 +435,7 @@ func newConfig(options ...configOption) (*Config, error) {
 		"bitwarden":                c.bitwardenTemplateFunc,
 		"bitwardenAttachment":      c.bitwardenAttachmentTemplateFunc,
 		"bitwardenFields":          c.bitwardenFieldsTemplateFunc,
+		"comment":                  c.commentTemplateFunc,
 		"decrypt":                  c.decryptTemplateFunc,
 		"encrypt":                  c.encryptTemplateFunc,
 		"fromToml":                 c.fromTomlTemplateFunc,
@@ -461,11 +463,13 @@ func newConfig(options ...configOption) (*Config, error) {
 		"onepasswordDetailsFields": c.onepasswordDetailsFieldsTemplateFunc,
 		"onepasswordDocument":      c.onepasswordDocumentTemplateFunc,
 		"onepasswordItemFields":    c.onepasswordItemFieldsTemplateFunc,
+		"onepasswordRead":          c.onepasswordReadTemplateFunc,
 		"output":                   c.outputTemplateFunc,
 		"pass":                     c.passTemplateFunc,
 		"passFields":               c.passFieldsTemplateFunc,
 		"passRaw":                  c.passRawTemplateFunc,
 		"quoteList":                c.quoteListTemplateFunc,
+		"replaceAllRegex":          c.replaceAllRegexTemplateFunc,
 		"secret":                   c.secretTemplateFunc,
 		"secretJSON":               c.secretJSONTemplateFunc,
 		"stat":                     c.statTemplateFunc,
@@ -775,12 +779,15 @@ func (c *Config) createConfigFile(filename chezmoi.RelPath, data []byte) ([]byte
 	funcMap := make(template.FuncMap)
 	chezmoi.RecursiveMerge(funcMap, c.templateFuncs)
 	initTemplateFuncs := map[string]interface{}{
-		"exit":          c.exitInitTemplateFunc,
-		"promptBool":    c.promptBoolInitTemplateFunc,
-		"promptInt":     c.promptIntInitTemplateFunc,
-		"promptString":  c.promptStringInitTemplateFunc,
-		"stdinIsATTY":   c.stdinIsATTYInitTemplateFunc,
-		"writeToStdout": c.writeToStdout,
+		"exit":             c.exitInitTemplateFunc,
+		"promptBool":       c.promptBoolInitTemplateFunc,
+		"promptBoolOnce":   c.promptBoolOnceInitTemplateFunc,
+		"promptInt":        c.promptIntInitTemplateFunc,
+		"promptIntOnce":    c.promptIntOnceInitTemplateFunc,
+		"promptString":     c.promptStringInitTemplateFunc,
+		"promptStringOnce": c.promptStringOnceInitTemplateFunc,
+		"stdinIsATTY":      c.stdinIsATTYInitTemplateFunc,
+		"writeToStdout":    c.writeToStdout,
 	}
 	chezmoi.RecursiveMerge(funcMap, initTemplateFuncs)
 
@@ -1392,6 +1399,7 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 		c.newEncryptCommand(),
 		c.newExecuteTemplateCmd(),
 		c.newForgetCmd(),
+		c.newGenerateCmd(),
 		c.newGitCmd(),
 		c.newIgnoredCmd(),
 		c.newImportCmd(),
@@ -1842,7 +1850,10 @@ func (c *Config) readLine(prompt string) (string, error) {
 	if err != nil {
 		return "", err
 	}
-	line, err := bufio.NewReader(c.stdin).ReadString('\n')
+	if c.bufioReader == nil {
+		c.bufioReader = bufio.NewReader(c.stdin)
+	}
+	line, err := c.bufioReader.ReadString('\n')
 	if err != nil {
 		return "", err
 	}
