@@ -15,17 +15,17 @@ func (c *Config) exitInitTemplateFunc(code int) string {
 	panic(chezmoi.ExitCodeError(code))
 }
 
-func (c *Config) promptBoolInitTemplateFunc(field string, args ...bool) bool {
+func (c *Config) promptBoolInitTemplateFunc(prompt string, args ...bool) bool {
 	switch len(args) {
 	case 0:
-		value, err := parseBool(c.promptStringInitTemplateFunc(field))
+		value, err := parseBool(c.promptStringInitTemplateFunc(prompt))
 		if err != nil {
 			panic(err)
 		}
 		return value
 	case 1:
-		promptStr := field + " (default " + strconv.FormatBool(args[0]) + ")"
-		valueStr := c.promptStringInitTemplateFunc(promptStr)
+		prompt += " (default " + strconv.FormatBool(args[0]) + ")"
+		valueStr := c.promptStringInitTemplateFunc(prompt)
 		if valueStr == "" {
 			return args[0]
 		}
@@ -40,16 +40,29 @@ func (c *Config) promptBoolInitTemplateFunc(field string, args ...bool) bool {
 	}
 }
 
-func (c *Config) promptIntInitTemplateFunc(field string, args ...int64) int64 {
+func (c *Config) promptBoolOnceInitTemplateFunc(m map[string]interface{}, key, prompt string, args ...bool) bool {
+	if len(args) > 1 {
+		err := fmt.Errorf("want 2 or 3 arguments, got %d", len(args)+2)
+		panic(err)
+	}
+	if value, ok := m[key]; ok {
+		if boolValue, ok := value.(bool); ok {
+			return boolValue
+		}
+	}
+	return c.promptBoolInitTemplateFunc(prompt, args...)
+}
+
+func (c *Config) promptIntInitTemplateFunc(prompt string, args ...int64) int64 {
 	switch len(args) {
 	case 0:
-		value, err := strconv.ParseInt(c.promptStringInitTemplateFunc(field), 10, 64)
+		value, err := strconv.ParseInt(c.promptStringInitTemplateFunc(prompt), 10, 64)
 		if err != nil {
 			panic(err)
 		}
 		return value
 	case 1:
-		promptStr := field + " (default " + strconv.FormatInt(args[0], 10) + ")"
+		promptStr := prompt + " (default " + strconv.FormatInt(args[0], 10) + ")"
 		valueStr := c.promptStringInitTemplateFunc(promptStr)
 		if valueStr == "" {
 			return args[0]
@@ -65,6 +78,19 @@ func (c *Config) promptIntInitTemplateFunc(field string, args ...int64) int64 {
 	}
 }
 
+func (c *Config) promptIntOnceInitTemplateFunc(m map[string]interface{}, key, prompt string, args ...int64) int64 {
+	if len(args) > 1 {
+		err := fmt.Errorf("want 2 or 3 arguments, got %d", len(args)+2)
+		panic(err)
+	}
+	if value, ok := m[key]; ok {
+		if intValue, ok := value.(int64); ok {
+			return intValue
+		}
+	}
+	return c.promptIntInitTemplateFunc(prompt, args...)
+}
+
 func (c *Config) promptStringInitTemplateFunc(prompt string, args ...string) string {
 	switch len(args) {
 	case 0:
@@ -90,6 +116,19 @@ func (c *Config) promptStringInitTemplateFunc(prompt string, args ...string) str
 	}
 }
 
+func (c *Config) promptStringOnceInitTemplateFunc(m map[string]interface{}, key, prompt string, args ...string) string {
+	if len(args) > 1 {
+		err := fmt.Errorf("want 2 or 3 arguments, got %d", len(args)+2)
+		panic(err)
+	}
+	if value, ok := m[key]; ok {
+		if stringValue, ok := value.(string); ok {
+			return stringValue
+		}
+	}
+	return c.promptStringInitTemplateFunc(prompt, args...)
+}
+
 func (c *Config) stdinIsATTYInitTemplateFunc() bool {
 	file, ok := c.stdin.(*os.File)
 	if !ok {
