@@ -7,6 +7,7 @@ import (
 	"os"
 	"os/exec"
 	"path/filepath"
+	"regexp"
 	"runtime"
 	"strconv"
 
@@ -22,6 +23,12 @@ type ioregData struct {
 	value map[string]interface{}
 }
 
+var startOfLineRx = regexp.MustCompile(`(?m)^`)
+
+func (c *Config) commentTemplateFunc(prefix, s string) string {
+	return startOfLineRx.ReplaceAllString(s, prefix)
+}
+
 func (c *Config) fromTomlTemplateFunc(s string) interface{} {
 	var data interface{}
 	if err := chezmoi.FormatTOML.Unmarshal([]byte(s), &data); err != nil {
@@ -167,6 +174,10 @@ func (c *Config) quoteListTemplateFunc(list []interface{}) []string {
 	return result
 }
 
+func (c *Config) replaceAllRegexTemplateFunc(expr, repl, s string) string {
+	return regexp.MustCompile(expr).ReplaceAllString(s, repl)
+}
+
 func (c *Config) statTemplateFunc(name string) interface{} {
 	switch fileInfo, err := c.fileSystem.Stat(name); {
 	case err == nil:
